@@ -1,10 +1,12 @@
 """
 Support functions for contexts that are also needed by modules which the
-`contexts.py` module depends on.
+`potluck.contexts` module depends on.
 
 context_utils.py
 """
 
+import io
+
 from . import html_tools
 
 
@@ -86,35 +88,57 @@ class ContextCreationError(ContextError):
         property of our cause if it's present and the cause is not a
         ContextCreationError.
         """
-        result = "Failed to establish context {}: {}".format(
-            self.context.feedback_topic(), # errors won't appear in rubric
-            self.message
+        result = "Could not get '{}' because:".format(
+            self.context.feedback_topic() # errors won't appear in rubric
         )
-        if self.cause and isinstance(self.cause, ContextCreationError):
-            # Short-circuit the extra builder failed step if we can:
-            if self.cause.message == "Builder failed." and self.cause.cause:
-                result += "<br>\n" + html_tools.build_html_details(
-                    "Caused by error in context {}:".format(
-                        self.cause.context.feedback_topic()
-                    ),
-                    getattr(self.cause.cause, "html_tb", str(self.cause))
+        if self.cause:
+            # Short-circuit to the root of the chain if we can
+            current = self
+            chain = []
+            while hasattr(current, "cause") and current.cause:
+                chain.append(current)
+                current = current.cause
+
+            # Add the root cause's message first
+            root_cause = current
+            result += "<br>\n" + str(root_cause)
+
+            # Try to get an HTML traceback for the root cause
+            root_tb = str(root_cause)
+            if hasattr(root_cause, "html_tb"):
+                root_tb = root_cause.html_tb
+            elif hasattr(root_cause, "__traceback__"):
+                if self.context.cached_value is not None:
+                    linkable = linkmap(self.context.cached_value)
+                elif self.context.working_from is not None:
+                    linkable = linkmap(self.context.working_from)
+                else:
+                    linkable = None
+                root_tb = html_tools.html_traceback(
+                    root_cause,
+                    linkable=linkable
                 )
-            else:
-                result += "<br>\n" + html_tools.build_html_details(
-                    "Caused by",
-                    self.cause.explanation()
+
+            result += html_tools.build_html_details(
+                "Details (click me):",
+                html_tools.build_list(
+                    [
+                        "{}: {}".format(
+                            error.context.feedback_topic(),
+                            error.message
+                        )
+                        for error in chain
+                    ] + [ root_tb ]
                 )
-        elif self.cause:
-            result += "<br>\n" + html_tools.build_html_details(
-                "Caused by",
-                getattr(self.cause, "html_tb", str(self.cause))
             )
+        else:
+            result += "<br>\n" + self.message
         return result
 
 
-#------------------#
-# Extract function #
-#------------------#
+#-------#
+# Utils #
+#-------#
 
 def extract(context, slot):
     """
@@ -129,6 +153,45 @@ def extract(context, slot):
         )
 
 
+def linkmap(context):
+    """
+    Returns a link dictionary suitable for the `linkable` parameter of
+    `html_traceback`, based on the "task_info" and "filename" slots of
+    the given context dictionary, or just based on the "task_info" if
+    "filename" is not present. Returns None if "task_info" is missing.
+    """
+    if "task_info" in context and "filename" in context:
+        return { context["filename"]: context["task_info"]["id"] }
+    elif "task_info" in context:
+        ti = context["task_info"]
+        return { ti["target"]: ti["id"] }
+    else:
+        return None
+
+
+class MashEnter(io.TextIOBase):
+    """
+    A fake input designed to be used in place of stdin. Reading anything
+    just returns newlines.
+    """
+    def read(size=-1):
+        """
+        Returns a newline, or if a specific size is specified, that many
+        newlines. Behavior is weird since normally you can't call read
+        multiple times.
+        """
+        if size == -1:
+            return '\n'
+        else:
+            return '\n' * size
+
+    def readline(size=-1):
+        """
+        Returns a newline.
+        """
+        return '\n'
+
+
 #---------------------------#
 # ContextualValue machinery #
 #---------------------------#
@@ -256,3 +319,38 @@ class ModuleValue(ContextualValue):
         module = context["module"]
 
         return eval(self.expression, module.__dict__)
+
+
+class SolnValue(ContextualValue):
+    """
+    Like a ModuleValue, but *always* takes the value from the solution
+    module, even when we're testing submitted code.
+    """
+    def __init__(self, expression):
+        """
+        One required argument: the expression to evaluate, which must be
+        a string that contains a valid Python expression.
+        """
+        self.expression = expression
+
+    def __str__(self):
+        return "the correct value of " + self.expression
+
+    def __repr__(self):
+        return "<a SolnValue based on {}>".format(str(self))
+
+    def replace(self, context):
+        """
+        We retrieve the "ref_module" slot value from the provided context.
+
+        We then evaluate our expression within the retrieved module, and
+        return that result.
+        """
+        if "ref_module" not in context:
+            raise MissingContextError(
+                f"SolnValue argument requires a 'ref_module' context"
+                f" key, but there isn't one in: {list(context.keys())}"
+            )
+        module = context["ref_module"]
+
+        return eval(self.expression, module.__dict__)
