@@ -117,14 +117,32 @@ class Context:
     include the following `potluck.context_utils.BASE_CONTEXT_SLOTS`:
 
     - "task_info": A task info dictionary for the current task, which
-        will have a "specification" key containing the specification
-        module for this task.
+        will have the following keys:
+        - "id": The task ID.
+        - "specification": the specification module for this task.
+        - "target": The default file or folder to grade in a submission.
+        - "title": The title of this task.
+        - "desc": The short description of this task.
+        - "reference_cache_file": The filename where reference values
+            may be cached.
+        - "ignore_cache": Whether or not to ignore the cache (might not
+            be present; ignore cache only if present and truthy).
     - "username": The user who submitted the code we're evaluating.
     - "submission_root": The root directory within which the
         submission we're evaluating can be found.
     - "default_file": The official name of the default file to evaluate.
     - "actual_file": The actual name of the default file as submitted.
 
+    When a context is associated with a specific goal, the context
+    dictionary will also always contain the following extra slots:
+
+    - "goal_id": The unique-per-task identifier string for the goal that
+        this context is associated with.
+    - "which_context": The index of this context within the goal's
+        testing contexts. Along with the task ID and goal ID, this can be
+        used to build a unique identifier for the context.
+        TODO: Context IDs?
+
     The typical slots of derived context dictionaries include:
 
     - "filename": The file name (without path) of the file we're
@@ -153,8 +171,27 @@ class Context:
     - "trace": a trace list recording certain function calls made
         during a test, along with arbitrary state snapshots (see
         `potluck.harness.tracing_function_calls`).
-    - "image_info": an image info dictionary that include a filename
-        which points to an image generated by a function call.
+    - "image": a Pillow Image object holding a captured image.
+    - "audio": a dictionary with "mimetype", "data", and optionally
+        "label" slots, holding captured audio data in binary form. The
+        mimetype should indicate a MIME type for the data, while the
+        "data" slot holds a bytes object containing the raw data in that
+        format. The label slot holds a string used to label the audio in
+        output to users.
+    - "expectations": a list of dictionaries defining expectations
+        established using the `optimism` module (only present if that
+        module is available) Each has the following keys:
+        - "tag" A string indicating the file name and line number where
+            this expectation was established
+        - "case" The test case info. This is a dictionary with "result",
+            "output", and "context" slots, holding the result value,
+            printed output, and context details for a test case (see
+            `optimism.get_my_context` for details of the "context"
+            value).
+        - "type" The type of expectation: "result", "output", or
+            "custom".
+        - "value" The expected value (or output fragment, or the checker
+            function, depending on the expectation type).
     - Other keys not defined here may be established and/or used by
         certain Context or Goal classes.
     - Versions of the keys above with "ref_" as a prefix hold
@@ -1273,10 +1310,13 @@ class ModuleContext(AutoContext):
         loaded module.
         """
         loaded = context[self._to]
-        tests = {
-            id(exp["case"]): exp["case"]
-            for exp in context[self._to_expectations]
-        }
+        if OPTIMISTIC:
+            tests = {
+                id(exp["case"]): exp["case"]
+                for exp in context[self._to_expectations]
+            }
+        else:
+            tests = None
         defined = [
             name
             for name in dir(loaded)
@@ -1427,7 +1467,10 @@ class ModuleContext(AutoContext):
 
         # Finally, register ourselves as an auto provider for the slots
         # that we generate:
-        self.register(self._to)
+        if OPTIMISTIC:
+            self.register(self._to, self._to_expectations)
+        else:
+            self.register(self._to)
 
 
 # Register a factory for a default ModuleContext as an on-demand option
@@ -1890,6 +1933,10 @@ random chance.):
     return display_distribution_results
 
 
+#---------------#
+# SiftedContext #
+#---------------#
+
 class SiftedContext(Context):
     """
     Working from the "output" and "ref_output" slots (or some other
